/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <netdb.h>
#include "List.h"
#include "ConnectionList.h"
#include "DownloadList.h"
#include "ErrorList.h"
#include "HostList.h"
#include "OptionList.h"
#include "QueryList.h"
#include "ResultList.h"
#include "ShareList.h"
#include "StatList.h"
#include "UploadList.h"
#include "ListMaker.h"
#include "HagelslagInterface.h"

HagelslagInterface::HagelslagInterface()
{
	buf[0]	= 0;
	sock	= 0;
	port	= 0;
}

int HagelslagInterface::startHagelslag(char *path, int newPort)
{
	int		res;
	struct stat	junk;
	char		executable[300];

	port	= newPort;

	if ((!path) || (!path[0]))
	{

		res	= -1;
		if (!stat("/usr/bin/hagelslagd", &junk))
		{
			sprintf(executable, "/usr/bin/hagelslagd -p %d &", port);
			res	= system(executable);
		}

		if ( (res == -1) || (res == 127) )
		{
			if (!stat("/usr/local/bin/hagelslagd", &junk))
			{
				sprintf(executable, "/usr/local/bin/hagelslagd -p %d &",
						port);
				res	= system(executable);
			}
		}

		if ( (res == -1) || (res == 127) )
			return 0;
	}
	else
	{
		if (!stat(path, &junk))
		{
			sprintf(executable, "%s -p %d &", path, port);
			res	= system(executable);
			if ( (res == -1) || (res == 127) )
				return 0;
		}
		else
			return 0;
	}
	sleep (3);
	return 1;
}

int HagelslagInterface::stopHagelslag()
{
	if (!sendStr("quit\n"))
		return 0;

	close (sock);
	return 1;
}

int HagelslagInterface::disconnectHagelslag()
{
	if (!sendStr("holdon\n"))
		return 0;

	close (sock);
	return 1;
}

int HagelslagInterface::connectHagelslag()
{
	struct sockaddr_in	sa;
	struct hostent		*h;
	char			host[256];

	gethostname(host, 256);
	sa.sin_port		= htons(port);
	sa.sin_family		= PF_INET;
	if (!((h 		= gethostbyname(host))))
		return 0;
	sa.sin_addr.s_addr	= *(unsigned long int *)h->h_addr_list[0];
	if ((sock 		= socket(PF_INET, SOCK_STREAM, 0)) == -1)
	{
		fprintf(stderr, "HagelslagInterface::connectHagelslag() : socket() failed!!\n");
		return 0;
	}

	if (connect(sock, (struct sockaddr *)&sa, sizeof(sa)))
	{
		fprintf(stderr, "HagelslagInterface::connectHagelslag() : Connect failed!!\n");
		perror("");
		return 0;
	}
	readStr();
	if (!strstr(buf, PROMPT))
	{
		fprintf(stderr, "HagelslagInterface::connectHagelslag() : No prompt!!\n");
		close(sock);
		return 0;
	}
	return 1;
}

int HagelslagInterface::connectHagelslag(char *host, int atPort)
{
	struct sockaddr_in	sa;
	struct hostent		*h;

	if (!host)
		return 0;

	port			= atPort;
	sa.sin_port		= htons(port);
	sa.sin_family		= PF_INET;
	if (!((h 		= gethostbyname(host))))
		return 0;
	sa.sin_addr.s_addr	= *(unsigned long int *)h->h_addr_list[0];
	if ((sock 		= socket(PF_INET, SOCK_STREAM, 0)) == -1)
	{
		fprintf(stderr, "HagelslagInterface::connectHagelslag() : socket() failed!!\n");
		return 0;
	}

	if (connect(sock, (struct sockaddr *)&sa, sizeof(sa)))
	{
		fprintf(stderr, "HagelslagInterface::connectHagelslag() : Connect failed!!\n");
		perror("");
		return 0;
	}
	readStr();
	if (!strstr(buf, PROMPT))
	{
		fprintf(stderr, "HagelslagInterface::connectHagelslag() : No prompt!!\n");
		close(sock);
		return 0;
	}
	return 1;
}

List *HagelslagInterface::list(int listType, int offset)
{
	char	out[50];
	List	*l;

	switch (listType)
	{
		case	LIST_RESULTS	: 
					sprintf(out, "list results %d\n",
						offset);
					break;
		case	LIST_HOSTS	:
					sprintf(out, "list hosts %d\n",
						offset);
					break;
		case	LIST_DOWNLOADS	: sprintf(out, "list downloads\n");
					break;
		case	LIST_UPLOADS	: sprintf(out, "list uploads\n"); 
					break;
		case	LIST_CONNECTIONS:
					sprintf(out, "list connections %d\n",
						offset);
					break;
		case	LIST_SHARES	: sprintf(out, "list shares\n"); 
					break;
		case	LIST_OPTIONS	: sprintf(out, "list options\n"); 
					break;
		case	LIST_STATS	: sprintf(out, "list stats\n"); 
					break;
		case	LIST_QUERIES	: sprintf(out, "list queries\n"); 
					break;
		default		:
				return new ErrorList("Unknown ListType");
	}

	sendStr(out);
	readStr();

	if (!strncmp(buf, PROMPT, strlen(PROMPT)))
		l	= 0;
	else
	{
		l	= lm.makeList(listType, buf);
		/*
		 * Read the Prompt
		 */
		readStr();
		if (strncmp(buf, PROMPT, strlen(PROMPT)))
			fprintf(stderr, "HagelslagInterface::list() : No PROMPT ?\n");
	}	
	
	return l;
}

ErrorList *HagelslagInterface::set(int param, int val, char *arg)
{
	char		outbuf[256];

	switch(param)
	{
		case	SET_MAX_TTL		:	
				sprintf(outbuf, "set MAX_TTL %d\n", val);
				break;
		case	SET_DEFAULT_TTL		:	
				sprintf(outbuf, "set DEFAULT_TTL %d\n", val);
				break;
		case	SET_MAX_CONNECTIONS	:	
				sprintf(outbuf, "set MAX_CONNECTIONS %d\n", val);
				break;
		case	SET_ROUTING_TABLE_SIZE	:	
				sprintf(outbuf, "set ROUTING_TABLE_SIZE %d\n", val);
				break;
		case	SET_MAX_HOPS		:	
				sprintf(outbuf, "set MAX_HOPS %d\n", val);
				break;
		case	SET_MAX_UPLOADS		:	
				sprintf(outbuf, "set MAX_UPLOADS %d\n", val);
				break;
		case	SET_MAX_DOWNLOADS	:	
				sprintf(outbuf, "set MAX_DOWNLOADS %d\n", val);
				break;
		case	SET_MAX_RESULTS		:	
				sprintf(outbuf, "set MAX_RESULTS %d\n", val);
				break;
		case	SET_PORT		:	
				sprintf(outbuf, "set PORT %d\n", val);
				break;
		case	SET_SPEED		:	
				sprintf(outbuf, "set SPEED %d\n", val);
				break;
		case	SET_DOWNLOAD_DIR	:	
				sprintf(outbuf, "set DOWNLOAD_DIR %s\n", arg);
				break;
		case	SET_AUTO_PUSH		:	
				sprintf(outbuf, "set AUTO_PUSH %d\n", val);
				break;
		case	SET_AUTO_CLEAR_COMPLETE	:	
				sprintf(outbuf, "set AUTO_CLEAR_COMPLETE %d\n", val);
				break;
		case	SET_AUTO_CONNECTIONS	:	
				sprintf(outbuf, "set AUTO_CONNECTIONS %d\n", val);
				break;
		case	SET_MAX_MONITOR_LENGTH	:	
				sprintf(outbuf, "set MAX_MONITOR_LENGTH %d\n", val);
				break;
	}

	sendStr(outbuf);
	readStr();
	return lm.makeErrorList(buf);
}

ErrorList *HagelslagInterface::host(int doWhat, int who, char *arg)
{
        char    out[256];

	switch (doWhat)
	{
		case	HOST_ADD	:
					sprintf(out, "host add %s\n", arg);
					break;
		case	HOST_DEL	:
					sprintf(out, "host del %d\n", who);
					break;
		case	HOST_CONNECT	:
					sprintf(out, "host connect %d\n", who);
					break;
		default		: return 0;
	}

	sendStr(out);
	readStr();
	return lm.makeErrorList(buf);
}

ErrorList *HagelslagInterface::download(int how, int which)
{
	char	out[256];
	char	*method;

	switch(how)
	{
		case DOWNLOAD_HOW_START		: 
					method	= "start";
					break;
		case DOWNLOAD_HOW_CONTINUE	: 
					method	= "continue";
					break;
		case DOWNLOAD_HOW_PUSH		:
					method	= "push";
					break;
		default	:
				return 0;
	}
	sprintf(out, "download %s %d\n", method, which);
	sendStr(out);
	readStr();
	return lm.makeErrorList(buf);
}

ErrorList *HagelslagInterface::share(int doWhat, char *arg)
{
	char	out[256];

	switch(doWhat)
	{
		case	SHARE_ADD	:
				sprintf(out, "share add %s\n", arg);
					break;
		case	SHARE_DEL	: 
				sprintf(out, "share del %s\n", arg);
					break;
		case	SHARE_CLEAR	: sprintf(out, "share clear\n");
					break;
		default	: return 0;
	}

	sendStr(out);
	readStr();
	return lm.makeErrorList(buf);
}

ErrorList *HagelslagInterface::save(int what)
{
	switch (what)
	{
		case SAVE_OPTIONS	: sendStr("save options\n");
						break;
		case SAVE_HOSTS		: sendStr("save hosts\n");
						break;
		case SAVE_SHARES	: sendStr("save shares\n");
						break;
	}

	readStr();
	return lm.makeErrorList(buf);
}

ErrorList *HagelslagInterface::kill(int what, int which)
{
	char	str[100];

	switch (what)
	{
		case KILL_CONNECTION	: sprintf(str, "kill connection %d\n", 
							which);
						break;
		case KILL_UPLOAD	: sprintf(str, "kill upload %d\n",
							which);
						break;
		case KILL_DOWNLOAD	: sprintf(str, "kill download %d\n",
							which); 
						break;
	}

	sendStr(str);
	readStr();
	return lm.makeErrorList(buf);
}

ErrorList *HagelslagInterface::scan()
{
	sendStr("scan\n");
	readStr();
	return lm.makeErrorList(buf);
}

ErrorList *HagelslagInterface::search(int doWhat, int speed, char *arg)
{
	if (doWhat == SEARCH_CLEAR)
		sendStr("search clear\n");
	else
	{
		char	out[300];

		sprintf(out, "search %d %s\n", speed, arg);
		sendStr(out);
	}

	readStr();
	return lm.makeErrorList(buf);
}

int HagelslagInterface::sendStr(char *arg)
{
	if (send(sock, arg, strlen(arg), 0) < 0)
		return 0;

	return 1;
}

int HagelslagInterface::readStr()
{
	unsigned int	res;

	if ((res = recv(sock, buf, BUFLEN, 0)) < 0)
	{
		perror("HagelslagInterface::readStr() ");
		return 0;
	}
	if (res < BUFLEN - 1)
		buf[res + 1]	= 0;
	else
		buf[BUFLEN - 1]	= 0;

	return res;
}

void HagelslagInterface::getHagelslagVersion(char *ver)
{
	char	*finder;

	if (!sendStr("version\n"))
	{
		*ver	= 0;
		return;
	}

	if (!readStr())
	{
		*ver	= 0;
		return;
	}
	
	if ((finder = strstr(buf, "\r\n")))
		*finder		= 0;
	else
	{
		*ver	= 0;
		return;
	}

	strcpy(ver, buf);
}

void HagelslagInterface::getApiVersion(char *ver)
{
	sprintf(ver, API_VERSION);
}

List *HagelslagInterface::listAllSomething(int what, int howMany, int step)
{
	int	i;
	List	*l, *prv;

	l	= 0;	
	for(i = 0; i < howMany; i += step)
	{
		prv	= l;
		l	= list(what, i);
		if (l)
		{
			List	*tmp;
			
			for (tmp = l; tmp->getNext(); tmp = tmp->getNext())
				;
			tmp->setNext(prv);
		}
		else
			l	= prv;		
	}
	
	return l;
}

List *HagelslagInterface::listAllHosts()
{
	return listAllSomething(LIST_HOSTS, 30000, 20);
}

List *HagelslagInterface::listAllConnections()
{
	return listAllSomething(LIST_CONNECTIONS, 150, 10);
}

List *HagelslagInterface::listAllResults()
{
	return listAllSomething(LIST_RESULTS, 3000, 10);
}

