/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include "List.h"
#include "ConnectionList.h"
#include "DownloadList.h"
#include "ErrorList.h"
#include "HostList.h"
#include "OptionList.h"
#include "QueryList.h"
#include "ResultList.h"
#include "ShareList.h"
#include "StatList.h"
#include "UploadList.h"
#include "ListMaker.h"
#include "HagelslagInterface.h"

List *ListMaker::makeList(int listType, char *buf)
{
	if (!strncmp(PROMPT, buf, strlen(PROMPT)))
		buf	+= strlen(PROMPT);

	switch (listType)
	{
		case	LIST_RESULTS	:
				return makeResultList(buf);
				break;
		case	LIST_DOWNLOADS	:
				return makeDownloadList(buf);
				break;
		case	LIST_UPLOADS	:
				return makeUploadList(buf);
				break;
		case	LIST_CONNECTIONS	:
				return makeConnectionList(buf);
				break;
		case	LIST_SHARES	:
				return makeShareList(buf);
				break;
		case	LIST_HOSTS		:
				return makeHostList(buf);
				break;
		case	LIST_OPTIONS	:
				return makeOptionList(buf);
				break;
		case	LIST_STATS		:
				return makeStatList(buf);
				break;
		case	LIST_QUERIES	:
				return makeQueryList(buf);
				break;
		default	: break;
	}

	return 0;
}

List *ListMaker::makeResultList(char *buf)
{
	List	*l, *prv;
	int	id, size, speed, fid;
	char	fileName[256];
	char	host[256];

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\r', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 2;
		}
		else
		{
			int	i;

			sscanf(buf, "%d :\t", &id);
			buf	= strstr(buf, " :\t");
			buf	+= strlen(" :\t");

			for (i = 0; strncmp(buf + i, "\r\n\tFileIndex: ",
						strlen("\r\n\tFileIndex: ")); i++)
				fileName[i]	= buf[i];

			fileName[i]	= 0;
			buf		+= i + strlen("\r\n\tFileIndex: ");

			sscanf(buf, "%d\tSize: %d\tHost: %s\tSpeed: %d.\r\n",
					&fid, &size, host, &speed);
			fileName[255]	= 0;
			host[24]	= 0;
			l	= new ResultList(id, fileName, fid,
					size, host, speed);
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
		{
			fprintf(stderr, "ListMaker::makeResultList() :breek!\n");
			break;
		}

		buf++;
	}
	
	return l;
}

List *ListMaker::makeDownloadList(char *buf)
{
	List	*l, *prv;
	int	id, fid, db, tb, ks;
	char	fileName[256];
	char	state[256];
	char	host[256];

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			int i;

			/*
			 * The string looks like this :
			 * "%d\t: %s ---> %s\r\n\tState: %s\t"
                         *       "FileID: %d\t%d / %d bytes %d.%d KB/s.\r\n"
			 */

			sscanf(buf, "%d\t: %s ---> ", &id, host);
			buf	= strstr(buf, " ---> ");
			buf	+= strlen(" ---> ");

			for (i = 0; strncmp(buf + i, "\r\n\tState: ",
						strlen("\r\n\tState: ")); i++)
				fileName[i]	= buf[i];

			fileName[i]	= 0;
			buf		+= i + strlen("\r\n\tState: ");

			sscanf(buf, "%s\tFileID: %d\t%d / %d bytes %d.%d KB/s.\r\n",
				state, &fid, &db, &tb, &ks, &i);
			ks	= ks * 1024 + i;
			l	= new DownloadList(id, host, fileName,
					state, fid, db, tb, ks);
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;

		buf++;
	}

	return l;
}

List *ListMaker::makeUploadList(char *buf)
{
	List	*l, *prv;
	int	id, fid, db, tb, ks;
	char	fileName[256];
	char	state[256];
	char	host[256];

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			int i;

			/*
			 * The string looks like this :
			 * "%d\t: %s ---> %s\r\n\tState: %s\t"
                         *       "FileID: %d\t%d / %d bytes %d.%d KB/s.\r\n"
			 */

			sscanf(buf, "%d\t: %s ---> ", &id, host);
			buf	= strstr(buf, " ---> ");
			buf	+= strlen(" ---> ");

			for (i = 0; strncmp(buf + i, "\r\n\tState: ", 
						strlen("\r\n\tState: ")); i++)
				fileName[i]	= buf[i];

			fileName[i]	= 0;
			buf		+= i + strlen("\r\n\tState: ");

			sscanf(buf, "%s\tFileID: %d\t%d / %d bytes %d.%d KB/s.\r\n",
				state, &fid, &db, &tb, &ks, &i);
			ks	= ks * 1024 + i;
			l	= new DownloadList(id, host, fileName,
					state, fid, db, tb, ks);
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;
		buf++;
	}

	return l;
}

List *ListMaker::makeConnectionList(char *buf)
{
	List	*l, *prv;
	int	id, sent, recvd, dropped, out, cid;
	char	host[256];
	char	state[256];
	char	strCID[50];
	char	strOUT[50];

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			sscanf(buf, "%d : %s\tSTATE: %s\tCID: %s\r\n"
				"\tSENT: %d\tRECEIVED: %d\tDROPPED: %d\t"
				"OUTGOING: %s\r\n",
				&id, host, state, strCID, &sent, &recvd,
				&dropped, strOUT);

			// Translate bools...
			cid	= strncmp(strCID, "NO", 2);
			out	= strncmp(strOUT, "NO", 2);

			l	= new ConnectionList(id, host, state,
					sent, recvd, dropped, out, cid);

			if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
				break;
			buf++;
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;

		buf++;
	}

	return l;
}

List *ListMaker::makeShareList(char *buf)
{
	List	*l, *prv;

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			char	path[512];

			sscanf(buf, "%s\r\n", path);
			path[255]	= 0;
			l	= new ShareList(path);
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;
		buf++;
	}

	return l;
}

List *ListMaker::makeHostList(char *buf)
{
	List	*l, *prv;

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			int	id, real, cid, tries;
			char	host[256];
			char	strReal[50], strCID[50];

			sscanf(buf, "%d\tHost: %s\tREAL: %s\tCID: %s\tTries: %d\r\n",
				&id, host, strReal, strCID, &tries);

			// translate bools
			real	= strncmp(strReal, "NO", 2);
			cid	= strncmp(strCID, "NO", 2);

			l	= new HostList(id, host, real, cid, tries);
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;

		buf++;
	}

	return l;
}

List *ListMaker::makeOptionList(char *buf)
{
	List	*l, *prv;

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			int	mttl, dttl, rts, mc, mr, mh, mpr, mu, md,
				mhost, p, spd, mml, ap, acc, ac, dp;
			char	dd[512];

			sscanf(buf, "MAX_TTL %d\r\nDEFAULT_TTL %d\r\n"
                                "ROUTING_TABLE_SIZE %d\r\nMAX_CONNECTIONS %d\r\n"
                                "MAX_RESULTS %d\r\nMAX_HOPS %d\r\n"
                                "MAX_PUSH_REQUESTS %d\nMAX_UPLOADS %d\r\n"
                                "MAX_DOWNLOADS %d\r\nMAX_HOSTS %d\r\n"
                                "PORT %d\r\nSPEED %d\r\n"
                                "MAX_MONITOR_LENGTH %d\r\n"
                                "DOWNLOAD_DIR %s\r\nAUTO_PUSH %d\r\n"
                                "AUTO_CLEAR_COMPLETE %d\r\nAUTO_CONNECTIONS %d\r\n"
                                "DAEMON_PORT %d\r\n", &mttl, &dttl, &rts, &mc,
				&mr, &mh, &mpr, &mu, &md, &mhost, &p, &spd,
				&mml, dd, &ap, &acc, &ac, &dp);

			l	= new OptionList(mttl, dttl, rts, mc, mr,
					mh, mpr, mu, md, mhost, p, spd,
					mml, ap, acc, ac, dd, dp);
			if (!((buf	= strstr(buf, "DAEMON_PORT "))))
			{
				fprintf(stderr, "ListMaker::makeOptionList() No DAEMON_PORT!!\n");
				return l;
			}
		}
		l->setNext(prv);
		if (!((buf	= (char *)memchr(buf, '\n', strlen(buf)))))
			break;

		buf++;
	}

	return l;
}

List *ListMaker::makeStatList(char *buf)
{
	List	*l, *prv;

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			int	h, fon, son, fs, ss;

			sscanf(buf, "Hosts: %d\tFiles On Net: %d\t"
                       		"Size on Net: %d\r\n"
				"Files Shared: %d\tSize Shared: %d\r\n",
				&h, &fon, &son, &fs, &ss);

			l	= new StatList(h, fon, son, fs, ss);
			if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
				break;
			buf++;
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;

		buf++;
	}

	return l;
}

List *ListMaker::makeQueryList(char *buf)
{
	List	*l, *prv;

	l	= 0;
	while (buf[0] && memchr(buf, '\n', strlen(buf)))
	{
		prv	= l;
		if (!strncmp(buf, "Error : ", strlen("Error : ")))
		{
			char	*finder;

			buf		+= strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
			*finder		= 0;
			l		= new ErrorList(buf);
			buf		= finder + 1;
		}
		else
		{
			char	pattern[512];
			int	spd;

			sscanf(buf, "%d %s\r\n", &spd, pattern);
			pattern[255]	= 0;
			l	= new QueryList(spd, pattern);
		}
		l->setNext(prv);

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;
		buf++;
	}

	return l;
}

ErrorList *ListMaker::makeErrorList(char *buf)
{
        ErrorList    *l, *prv;

        l       = 0;
        while (buf[0] && memchr(buf, '\n', strlen(buf)))
        {
		prv	= l;
                if (!strncmp(buf, "Error : ", strlen("Error : ")))
                {
                        char    *finder;

                        buf             += strlen("Error : ");
			finder		= (char *)memchr(buf, '\n', strlen(buf));
                        *finder         = 0;
                        l               = new ErrorList(buf);
                        buf             = finder + 1;
			l->setNext(prv);
                }

		if (!((buf = (char *)memchr(buf, '\n', strlen(buf)))))
			break;

		buf++;
	}

	return l;
}
