/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <List.h>
#include <ConnectionList.h>
#include <DownloadList.h>
#include <ErrorList.h>
#include <HostList.h>
#include <OptionList.h>
#include <QueryList.h>
#include <ResultList.h>
#include <ShareList.h>
#include <StatList.h>
#include <UploadList.h>
#include <ListMaker.h>
#include <HagelslagInterface.h>
#include "ListTranslator.h"

int ListTranslator::listToStr(List *l, char *str)
{
	*str	= 0;

	while (l)
	{
		switch (l->getType())
		{
			case	LIST_TYPE_RESULTS	:
				ResultListToStr((ResultList *)l, str);
				break;
			case	LIST_TYPE_DOWNLOADS	:
				DownloadListToStr((DownloadList *)l, str);	
				break;
			case	LIST_TYPE_UPLOADS	:
				UploadListToStr((UploadList *)l, str);
				break;
			case	LIST_TYPE_CONNECTIONS	:
				ConnectionListToStr((ConnectionList *)l, str);
				break;
			case	LIST_TYPE_SHARES	:
				ShareListToStr((ShareList *)l, str);
				break;
			case	LIST_TYPE_HOSTS		:
				HostListToStr((HostList *)l, str);
				break;
			case	LIST_TYPE_OPTIONS	:
				OptionListToStr((OptionList *)l, str);
				break;
			case	LIST_TYPE_STATS		:
				StatListToStr((StatList *)l, str);
				break;
			case	LIST_TYPE_QUERIES	:
				QueryListToStr((QueryList *)l, str);
				break;
			case	LIST_TYPE_ERROR		:
				ErrorListToStr((ErrorList *)l, str);
				break;
		default:
				fprintf(stderr, "ListTranslator::listToStr() : Unknown LIST_TYPE\r\n");
				return 0;
		}
		str	+= strlen(str);
		l	= l->getNext();
	}

	return 1;
}

int ListTranslator::ConnectionListToStr(ConnectionList *l, char *str)
{
	char	host[50];
	char	state[50];

	l->getHost(host);
	l->getState(state);

	sprintf(str, "Id %d\tout ? %d\t cid ? %d\tHost : %s\r\n"
			"\tstate %s\t snt %d\t recv %d\t dr %d\r\n",
			l->getID(), l->isOutgoing(), l->hasKnownCID(),
			host, state, l->getSent(), l->getReceived(), 
			l->getDropped());
	return 1;
}

int ListTranslator::DownloadListToStr(DownloadList *l, char *str)
{
	char	host[50];
	char	state[50];
	char	fn[256];

	l->getHost(host);
	l->getState(state);
	l->getFileName(fn);

	sprintf(str, "Id %d\tfileId %d\tkbps %d\tState %s\r\n"
			"\tDB %d\t TB %d\t%s ---> %s\r\n",
			l->getID(), l->getFileIndex(), l->getKbps(),
			state, l->getDoneBytes(), l->getTotalBytes(),
			host, fn);
			
	return 1;
}

int ListTranslator::ErrorListToStr(ErrorList *l, char *str)
{
	l->getError(str);

	return 1;
}

int ListTranslator::HostListToStr(HostList *l, char *str)
{
	char	host[50];

	l->getHost(host);
	sprintf(str, "ID %d\treal ? %d\tcid ? %d\tHost %s\r\n",
		l->getID(), l->isReal(), l->clientIDIsKnown(), host);

	return 1;
}

int ListTranslator::OptionListToStr(OptionList *l, char *str)
{
	char	dd[256];

	l->getDownloadDir(dd);
	dd[255]		= 0;
	sprintf(str, "MAX_TTL %d\r\nDEFAULT_TTL %d\r\n"
			"ROUTING_TABLE_SIZE %d\r\nMAX_CONNECTIONS %d\r\n"
			"MAX_RESULTS %d\r\nMAX_HOPS %d\r\n"
			"MAX_PUSH_REQUESTS %d\r\nMAX_UPLOADS %d\r\n"
			"MAX_DOWNLOADS %d\r\nMAX_HOSTS %d\r\n"
			"PORT %d\r\nSPEED %d\r\n"
			"MAX_MONITOR_LENGTH %d\r\n"
			"DOWNLOAD_DIR %s\r\n"
			"AUTO_PUSH %d\r\n"
			"AUTO_CLEAR_COMPLETE %d\r\n"
			"AUTO_CONNECTIONS %d\r\n"
			"DAEMON_PORT %d\r\n",
			l->getMaxTTL(), l->getDefaultTTL(),
			l->getRoutingTableSize(), l->getMaxConnections(),
			l->getMaxResults(), l->getMaxHops(),
			l->getMaxPushRequests(), l->getMaxUploads(),
			l->getMaxDownloads(), l->getMaxHosts(),
			l->getPort(), l->getSpeed(),
			l->getMonitorLength(), dd,
			l->getAutoPush(), l->getAutoClearComplete(), 
			l->getAutoConnections(), l->getDaemonPort()
		);

	return 1;
}

int ListTranslator::QueryListToStr(QueryList *l, char *str)
{
	sprintf(str, "spd %d\t? ", l->getSpeed());
	str	+= strlen(str);

	l->getQuery(str);

	return 1;
}

int ListTranslator::ResultListToStr(ResultList *l, char *str)
{
	char	fn[256];
	char	host[30];

	l->getFileName(fn);
	l->getHost(host);

	sprintf(str, "Id %d\tspd %d\tsize %d\tfileIndex %d\r\n"
			"host %s\t fn %s\r\n",
			l->getID(), l->getSpeed(), l->getSize(),
			l->getFileIndex(), host, fn);
 
	return 1;
}

int ListTranslator::ShareListToStr(ShareList *l, char *str)
{
	l->getPath(str);

	return 1;
}

int ListTranslator::StatListToStr(StatList *l, char *str)
{
	sprintf(str, "Hon %d\tfon %d\tson %d\tfs %d\tss %d\r\n",
		l->getHostsOnNet(), l->getFilesOnNet(),
		l->getSizeOnNet(), l->getFilesShared(), 
		l->getSizeShared());

	return 1;
}

int ListTranslator::UploadListToStr(UploadList *l, char *str)
{
        char    host[50];
        char    state[50];
        char    fn[256];

        l->getHost(host);
        l->getState(state);
        l->getFileName(fn);

        sprintf(str, "Id %d\tfileId %d\tkbps %d\tState %s\r\n"
                        "\tDB %d\t TB %d\t%s ---> %s\r\n",
                        l->getID(), l->getFileIndex(), l->getKbps(),
                        state, l->getDoneBytes(), l->getTotalBytes(),
                        fn, host);

        return 1;
}



