/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include "ControlCentre.h"
#include "Config.h"
#include "DBElement.h"
#include "Buffer.h"
#include "GnutellaPacket.h"
#include "Connection.h"

Connection::Connection()
{
	connection	= -1;
	hostIndex	= -1;
	type		= CON_TYPE_INVALID;
	state		= CON_STATE_INACTIVE;
	lock		= 0;
	outgoing	= 0;
	tries		= 0;
	timeStamp	= time(0);
	inBuffer	= new Buffer(CONNECTION_BUF_SIZE);
	outBuffer	= new Buffer(CONNECTION_BUF_SIZE);
}

Connection::Connection(int newConnection)
{
	connection	= newConnection;
	hostIndex	= -1;
	type		= CON_TYPE_INVALID;
	state		= CON_STATE_INACTIVE;
	lock		= 0;
	outgoing	= 0;
	tries		= 0;
	timeStamp	= time(0);
	inBuffer	= new Buffer(CONNECTION_BUF_SIZE);
	outBuffer	= new Buffer(CONNECTION_BUF_SIZE);
	fcntl(connection, F_SETFL, O_NONBLOCK);
}

Connection::Connection(Connection *c)
{
	connection	= c->getConnection();
	hostIndex	= c->getHostIndex();
	type		= c->getType();
	state		= c->getState();
	lock		= c->getLock();
	outgoing	= c->isOutgoing();
	tries		= c->getTries();
	timeStamp	= c->getTimeStamp();
	inBuffer	= c->copyInBuffer();
	outBuffer	= c->copyOutBuffer();
	fcntl(connection, F_SETFL, O_NONBLOCK);
}

Connection::Connection(int newConnection, struct sockaddr_in newsa)
{
	connection	= newConnection;
	hostIndex	= -1;
	type		= CON_TYPE_INVALID;
	state		= CON_STATE_INACTIVE;
	lock		= 0;
	sa		= newsa;
	outgoing	= 0;
	tries		= 0;
	timeStamp	= time(0);
	inBuffer	= new Buffer(CONNECTION_BUF_SIZE);
	outBuffer	= new Buffer(CONNECTION_BUF_SIZE);
	fcntl(connection, F_SETFL, O_NONBLOCK);
}

Connection::Connection(int newConnection, int newHostIndex)
{
	connection	= newConnection;
	hostIndex	= newHostIndex;
	type		= CON_TYPE_INVALID;
	state		= CON_STATE_INACTIVE;
	lock		= 0;
	outgoing	= 0;
	tries		= 0;
	timeStamp	= time(0);
	inBuffer	= new Buffer(CONNECTION_BUF_SIZE);
	outBuffer	= new Buffer(CONNECTION_BUF_SIZE);
	fcntl(connection, F_SETFL, O_NONBLOCK);
}

Connection::~Connection()
{
	delete inBuffer;
	delete outBuffer;
}

int Connection::isOutgoing()
{
	return outgoing;
}

void Connection::setOutgoing()
{
	outgoing	= 1;
}

int Connection::getState()
{
	return state;
}

void Connection::setState(int newState)
{
	state	= newState;
}

int Connection::getTries()
{
	return tries;
}

Buffer *Connection::copyInBuffer()
{
	return inBuffer->copyBuffer();
}

Buffer *Connection::copyOutBuffer()
{
	return outBuffer->copyBuffer();
}

int Connection::bufferSend()
{
	int	i;
	
	if (((i = outBuffer->bufSend(connection))) == -1)
		setState(CON_STATE_CON_ERROR);

	return i;
}

int Connection::sendStr(char *buf, unsigned int buflen)
{
	if (outBuffer->bytesEmpty() < buflen)
		return 0;

	return outBuffer->add(buf, buflen);
}

int Connection::read()
{
	return inBuffer->bufRecv(connection);
}

void Connection::setLock(int i)
{
	lock	= i;
}

int Connection::getLock()
{
	return lock;
}

int Connection::getInBuffer(char *buf, unsigned int range)
{
	return inBuffer->bufSuck(buf, range);
}

int Connection::peekInBuffer(char *buf, unsigned int range)
{
	return inBuffer->bufPeek(buf, range);
}		

unsigned int Connection::getInBufferSize()
{
	return inBuffer->getSize();
}

unsigned int Connection::getOutBufferSize()
{
	return outBuffer->getSize();
}

unsigned int Connection::getInBufferBytes()
{
	return inBuffer->bytesInBuffer();
}

unsigned int Connection::getOutBufferBytes()
{
	return outBuffer->bytesInBuffer();
}

int Connection::getConnection()
{
	return connection;
}

int Connection::getType()
{
	return type;
}

int Connection::getHostIndex()
{
	return hostIndex;
}

int Connection::skipBytes(unsigned int bytes)
{
	return inBuffer->skip(bytes);
}

void Connection::setConnection(int newConnection)
{
	connection	= newConnection;
	fcntl(connection, F_SETFL, O_NONBLOCK);
}

void Connection::setHostIndex(int newHostIndex)
{
	hostIndex	= newHostIndex;
}

void Connection::closeConnection()
{
	close(connection);
}

void Connection::setType(int newType)
{
	type	= newType;
}

int Connection::tryToConnect()
{
        fd_set  	out_set;
	struct timeval	tm;
	int		opt;
	socklen_t	len;

	if (state == CON_STATE_INACTIVE)
	{
		if (!connect(connection, (struct sockaddr *)&sa, sizeof(sa)))
		{
			resetTimeStamp();
			setState(CON_STATE_CONNECTED);
			return  1;
		}
		else
		{
			setState(CON_STATE_INIT);
			return 0;
		}
	}

	if (state != CON_STATE_INIT)
		return 0;

	tm.tv_sec	= 0;
	tm.tv_usec	= 1;
        FD_ZERO(&out_set);
        FD_SET(connection, &out_set);

        switch (select(connection + 1, 0, &out_set, 0, &tm))
	{
		case	-1	: break;
		case	 0	: break;
		default		: len	= (socklen_t)sizeof(opt);
				if (getsockopt(connection, SOL_SOCKET, SO_ERROR, &opt, &len))
				{
					setState(CON_STATE_CON_FAILED);
					return -1;
				}
				if (opt)
				{
					setState(CON_STATE_CON_FAILED);
					return -1;
				}
				resetTimeStamp();
				setState(CON_STATE_CONNECTED);
				return  1;
	}

	if (getAge() > CONNECTION_TIMEOUT)
	{
		setState(CON_STATE_CON_FAILED);
		return -1;
	}

	return 0;
}

time_t Connection::getTimeStamp()
{
	return timeStamp;
}

void Connection::resetTimeStamp()
{
	timeStamp	= time(0);
}

time_t Connection::getAge()
{
	return	time(0) - timeStamp;
}

/*
 * DBElement Inheritance
 */

int Connection::equals(DBElement * con)
{
	return (connection == ((Connection *)con)->getConnection());
}

int Connection::lessThan(DBElement * con)
{
	return (connection < ((Connection *)con)->getConnection());
}

int Connection::greaterThan(DBElement * con)
{
	return (connection > ((Connection *)con)->getConnection());
}

unsigned int Connection::hashValue()
{
	return (unsigned int)(connection & 0x1FFFF);
}

/*
 * FAME
 */
char *Connection::getInBufferp(unsigned int range)
{
	return inBuffer->bufSuckp(range);
}
