/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "ControlCentre.h"
#include "DBElement.h"
#include "Buffer.h"
#include "GnutellaPacket.h"
#include "Connection.h"
#include "Transfer.h"
#include "Download.h"
#include "DB.h"
#include "Host.h"
#include "TransferStatus.h"
#include "PushRequest.h"
#include "Downloader.h"

Downloader::Downloader(ControlCentre *newMCC)
{
	MCC		= newMCC;
	downloads	= new DB(MCC->get_MAX_DOWNLOADS());
	pushRequests	= new DB(MCC->get_MAX_PUSH_REQUESTS());
}

Downloader::~Downloader()
{
	downloads->clearDB();
	pushRequests->clearDB();
	delete downloads;
	delete pushRequests;
}

int Downloader::newPushRequest(int fid, char *fn, char *mid, char *cid, int os)
{
	PushRequest		*p;

	if (pushRequests->isFull())
		return 0;

	p	= new PushRequest(fid, fn, mid, cid, os);

	if (pushRequests->storeElement(p) == ENTRY_INVALID)
	{
		delete(p);
		return 0;
	}

	return 1;
}

int Downloader::pollDownloader()
{
	Download	*d;
	int		i;
	int		res;

	if (!downloads->getElements())
		return 0;

	res	= 0;
	for (i = 0; i < downloads->getSize(); i++)
		if ((d = (Download *)downloads->findElement(i)))
		{
			switch (d->getState())
			{
				case CON_STATE_INIT :
						if (d->tryToConnect())
							res	= 1;
						break;
				case CON_STATE_CONNECTED	:
						if (d->getType() == CON_TYPE_PUSH_IN)
						{
							if (handlePendingPush(d))
							{
								d->Start(0);
								d->loginToPeer();
								res	= 1;
							}
						}
						else if (d->loginToPeer())
							res	= 1;
						break;
				case CON_STATE_LOGIN_SENT	:
					if (d->getType() == CON_TYPE_DOWNLOAD)
					{
						if (d->getOutBufferBytes() > 0)
						{
							d->bufferSend();
							break;
						}
						else
						{
							d->read();
							d->waitResponse();
							break;
						}
					}
					if (d->getType() == CON_TYPE_PUSH_IN)
						d->setState(CON_STATE_ACTIVE);
					break;
				case CON_STATE_ACTIVE	:
						if (d->readBlob())
							res	= 1;
						break;
				default		:
						break;
			}
		}
	clearComplete();

	return res;
}

int Downloader::startDownload(unsigned long int netOrderIP, int port, 
				int id, char *fn, char *cid)
{
	char	dirName[FILE_NAME_LENGTH];
	char	fileName[FILE_NAME_LENGTH];

	if (downloads->isFull())
	{
		MCC->addError("Downloader::startDownload : Downloads = full");
		return 0;
	}

	MCC->get_DOWNLOAD_DIR(dirName);
	sprintf(fileName, "%s/%s", dirName, fn);
	remove(fileName);
	return continueDownload(netOrderIP, port, id, fn, cid);
}

int Downloader::continueDownload(unsigned long int netOrderIP, int port,
					int id, char *fn, char *cid)
{
	Download		*d;
	struct 	sockaddr_in	sa;
	struct	stat		statData;
	int			offset;
	int			sock;
	char			downloadDir[FILE_NAME_LENGTH];
	char			fileName[2 * FILE_NAME_LENGTH];
	char			fn2[2 * FILE_NAME_LENGTH];
	char			*c;

	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		MCC->addError("Downloader : Unable to open socket!!!");
		return 0;
	}

	strcpy(fn2, fn);
	while ((c = strchr(fn2, '/')))
		*c	= '_';
	MCC->get_DOWNLOAD_DIR(downloadDir);
	sprintf(fileName, "%s/%s", downloadDir, fn2);
	fn2[FILE_NAME_LENGTH - 1]	= 0;
	if (stat(fileName, &statData) == -1)
		offset	= 0;
	else
		offset	= statData.st_size;

	sa.sin_family		= PF_INET;
	sa.sin_port		= htons(port);
	sa.sin_addr.s_addr	= netOrderIP;

        sock    = socket(PF_INET, SOCK_STREAM, 0);
  	d	= new Download(sock, id, fn2, downloadDir, offset, sa, cid);
	d->setType(CON_TYPE_DOWNLOAD);
	d->tryToConnect();

	if (downloads->storeElement(d) == ENTRY_INVALID)
	{
		MCC->addError("Downloader : store failed!");
		delete d;
		return 0;
	}	

	if ( !d->Start(0))
	{
		MCC->addError("Downloader : start failed!");
		downloads->deleteElement(d->getEntry());
		return 0;
	}

	return 1;
}

int Downloader::killDownload(int i)
{
	return downloads->deleteElement(i);
}

TransferStatus *Downloader::getStatusList()
{
	TransferStatus	*prv, *current;
	int		i;

	current		= 0;
	for (i = 0; i < downloads->getSize(); i++)
	{
		Download	*d;

		if ((d = (Download *)downloads->findElement(i)))
		{
			prv	= current;
			current	= new TransferStatus(d);
			current->setNext(prv);
		}
	}
	return current;
}

int Downloader::handlePendingPush(Download *d)
{
	int			bs;
	int			hisFID;
	char			hisFName[FILE_NAME_LENGTH + 10];
	char			hisMID[MESSAGE_ID_LENGTH];
	char			hisCID[CLIENTID_LENGTH];
	char			request[512];
	char			pn[FILE_NAME_LENGTH];
	char			fn[FILE_NAME_LENGTH];
	char			*ptr;	
	PushRequest		*finder;
	PushRequest		*p;
	int			index;
	int			i;
	int			CIDmatch;
	char			cid[CLIENTID_LENGTH];

	d->read();
	if (d->getInBufferBytes() <
		strlen("GIV x:ABCDEFGHIJKLMNOPQRSTUVWXYZ012345/y\r\n"))
					// Client ID in ascii armor = 32 bytes
	{
		if (d->getAge()	> 30)
			d->setState(CON_STATE_LOGIN_FAILED);
		return 0;
	}

	bs	= d->peekInBuffer(request, 511);
	request[bs]	= 0;
	
	if (!strstr(request, "\n\n"))
	{
		if (d->getAge()	> 30)
			d->setState(CON_STATE_LOGIN_FAILED);
		return 0;
	}
	sscanf(request, "GIV %d", &hisFID);
	if (!((ptr = strchr(request, ':'))))
	{
		d->setState(CON_STATE_LOGIN_FAILED);
		MCC->addError("Downloader : handlePendingPush : no ':' found!");
		return 0;
	}
	
	ptr++; 		// skip ':'
	for (i = 0; i < 31; i += 2)
	{
		char	hex [] = {	'0', '1', '2', '3', '4', '5', '6', '7',
					'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
		hisCID[i / 2]	= ((strchr(hex, ptr[i]) - hex) * 0x10) +
					(strchr(hex, ptr[i + 1]) - hex);
	}
	ptr	+= 2 * CLIENTID_LENGTH;

	ptr++; 		// skip '/'
	for (i = 0; ( (i < FILE_NAME_LENGTH - 1) && (ptr[i] != '\n') &&
			(ptr[i] != '\r') ); i++)
		hisFName[i]	= ptr[i];

	hisFName[i]	= 0;
	ptr		+= strlen(hisFName);
	finder	= new PushRequest(hisFID, hisFName, hisMID, hisCID, 0);
	
	if ((index = pushRequests->containsElement(finder)) == ENTRY_INVALID)
	{
		d->setState(CON_STATE_LOGIN_FAILED);
		MCC->addError("Downloader : handlePendingPush : Pushrequest not found!");
		delete finder;
		return 0;
	}
	delete finder;

	// Get info from pushRequest

	if (!((p = (PushRequest *)pushRequests->findElement(index))))
	{
		d->setState(CON_STATE_LOGIN_FAILED);
		MCC->addError("Downloader : handlePendingPush : No such pushRequest!");
		return 0;
	}

	p->getFileName(fn);
	p->getClientID(cid);

	CIDmatch	= 1;
	for (i = 0; i < CLIENTID_LENGTH; ++i)
		if (cid[i] != hisCID[i])
		{
			CIDmatch	= 0;
			break;
		}

	// Make sure everything is as expected
	if (	(!CIDmatch)			||
		(hisFID != p->getFileID())	||
		(strcmp(fn, hisFName))		)
	{
		d->setState(CON_STATE_LOGIN_FAILED);
		MCC->addError("Downloader : handlePendingPush : Mismatch in push!");
		pushRequests->deleteElement(index);
		return 0;
	}
	pushRequests->deleteElement(index);

	MCC->get_DOWNLOAD_DIR(pn);
	d->reInit(hisFID, fn, pn, hisCID);	
	
	/*
	 * Remove Header
	 */
	if ((int)(ptr - request) > (int)strlen(request))
	{
		MCC->addError("Downloader : handlePendingPush : Couldn't remove header");
		d->skipBytes(strlen(request));
	}
	else
		d->skipBytes((int)(ptr - request));

        return 1;
}

int Downloader::handlePush(Connection *c)
{
	Download		*d;

	d	= new Download(c);
	delete c;

	d->setType(CON_TYPE_PUSH_IN);
	if (downloads->storeElement(d) == ENTRY_INVALID)
	{
		MCC->addError("Downloader : handlePush : store failed!");
		delete d;
		return 0;
	}

	return 1;
}

int Downloader::resize()
{
	DB	*newDownloads;
	DB	*newPushRequests;

	int	i;

	if (MCC->get_MAX_DOWNLOADS() < downloads->getElements())
	{
		MCC->addError("Too many downloads for "
				"resize!");
		return 0;
	}

	newDownloads	= new DB(MCC->get_MAX_DOWNLOADS());
	newPushRequests	= new DB(MCC->get_MAX_PUSH_REQUESTS());

	for (i = 0; (	(i < newDownloads->getSize())	&&
			(i < downloads->getSize())	); i++)
		newDownloads->storeElement(downloads->findElement(i));

	for (i = 0; (	(i < newPushRequests->getSize())	&&
			(i < pushRequests->getSize())	); i++)
		newPushRequests->storeElement(pushRequests->findElement(i));

	delete downloads;
	delete pushRequests;

	downloads	= newDownloads;
	pushRequests	= newPushRequests;

	return 1;
}

int Downloader::isUsingHost(int hi)
{
	int		i;
	Download	*d;

	for (i = 0; i < downloads->getSize(); i++)
	{
		if ((d = (Download *)downloads->findElement(i)))
			if (d->getHostIndex() == hi)
				return 1;
	}

	return 0;
}

DB *Downloader::getDownloads()
{
	return downloads;
}

int Downloader::clearComplete()
{
	int		i;
	Download	*d;

	if (!MCC->get_AUTO_CLEAR_COMPLETE())
		return 0;

	for (i = 0; i < downloads->getSize(); i++)
		if ((d = (Download *)downloads->findElement(i)))
		{
			switch (d->getState())
			{
				case CON_STATE_LOGIN_FAILED	:
				case CON_STATE_CON_ERROR	:
				case CON_STATE_DONE		:
				case CON_STATE_CANCEL		:
				case CON_STATE_CLOSED		:
				case CON_STATE_INVALID		:
					killDownload(i);
					break;
				default : break;
			}
		}
	return 1;
}
