/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netdb.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <ctype.h>
#include "ControlCentre.h"
#include "DBElement.h"
#include "DB.h"
#include "Buffer.h"
#include "GnutellaPacket.h"
#include "Connection.h"
#include "Packet.h"
#include "Host.h"
#include "Path.h"
#include "File.h"
#include "CacheEntry.h"
#include "Transfer.h"
#include "Upload.h"
#include "TransferStatus.h"
#include "LocalResult.h"
#include "FServer.h"

#define MKARRAY(A,X,T)	if (!A[X])						\
			{								\
				int	sz;						\
											\
				sz	= (files->getElements() / UIBITS) + 1;		\
 				A[X]	= (unsigned int *)calloc(sz, sizeof(unsigned int));	\
				T	+= sz;						\
				if (!A[X])						\
				{							\
					MCC->addError("Calloc failed!!!");		\
					return 0;					\
				}							\
  			}

FServer::FServer(ControlCentre *newMCC)
{
	int	i;

	MCC		= newMCC;
	uploads		= new DB(MCC->get_MAX_UPLOADS());
	if (MCC->get_CACHE_ENTRIES() > 0)
		cache		= new DB(MCC->get_CACHE_ENTRIES());
	else
		cache	= 0;
	files		= 0;
	sizeShared	= 0;
	paths		= 0;
	localSearches	= 0;
	cacheHits	= 0;
	searchArraySize	= 0;
	hashArraySize	= 0;
	strcpy(vc, VALIDCHARS);

	BITS[0]		= 1;
	for (i = 1; i < (int)UIBITS; ++i)
		BITS[i]	= 2 * BITS[i - 1];

	for (i = 0; i < (int)SEARCH_ARRAY_SIZE; ++i)
		searchArray[i]	= 0;

	for (i = 0; i < (int)HASH_ARRAY_SIZE; ++i)
		hashArray[i]	= 0;
}

Path *FServer::getPaths()
{
	return paths;
}

int FServer::addUpload(Connection *c)
{
	Upload			*u;
	char			request[512];
	int			fid;
	int			bytesRead;
	int			offset;
	unsigned int 		len;
	int			i;
	char			fn[FILE_NAME_LENGTH];
	char			*finder;
	File			*f;
	Path			*fpath;
	struct sockaddr_in	sa;

	if ((uploads->isFull()) || (c->getState() != CON_STATE_CONNECTED) || (!files))
	{
		delete c;
		return 0;
	}

	if ((bytesRead = c->peekInBuffer(request, 511)) < 12)
 		return 0;

	request[bytesRead]	= 0;

	if (!strstr(request, "\r\n\r\n"))
		return 0;

	finder	= &(request[0]);
	finder	+= strlen("GET /get/");
	fid	= atoi(finder);
	while (*finder != '/')
		finder++;
	finder++;
// FIXME : this could be done faster with a strstr construction!
	for (i = 0; strncmp(" HTTP/1.", finder + i,
					strlen(" HTTP/1.")); i++)
		fn[i]	= finder[i];
	fn[i]	= 0;
	if (!(f = (File *)files->findElement(fid)))
	{
		MCC->addError("FServer : file not found in shares!");
		c->closeConnection();
		delete c;
		return 0;
	}
	
	if (strncmp(fn, f->getFileNamep(), FILE_NAME_LENGTH))
	{
		// FIXME : should return a 404 doc not found thing.
		c->closeConnection();
		delete c;
		return 0;
	}
	// Everything seems fine. Check for a content-range / offset
	if ((finder = strstr(request, "Content-Range:")))
	{
		finder	+= strlen("Content-Range:");
		while (*finder == ' ')
			finder++;
		offset	= atoi(finder);
	}
	else
		offset	= 0;

	// Skip the HTTP header
	if (!(finder = strstr(request, "\n")))
	{
		MCC->addError("FServer : End of HTTP header not found!");
		c->closeConnection();
		delete c;
		return 0;
	}
	finder	+= strlen("\n");
	c->skipBytes((int)(finder - request));

	// Create an Upload and store it.
	fpath	= f->getPath();
	len	= sizeof(sa);
	getsockname(c->getConnection(), (struct sockaddr *)&sa, &len);
	u	= new Upload(c->getConnection(), fid, f->getFileNamep(),
				fpath->getPathp(), offset, sa, MCC->get_CLIENTIDp());
	u->setType(CON_TYPE_UPLOAD);
	if (!u->Start(1))
	{
		delete u;
		delete c;
		return 0;
	}
	
	u->setState(CON_STATE_CONNECTED);

	if (uploads->storeElement(u) == ENTRY_INVALID)
	{
		// This is theoretically impossible.
		MCC->addError("FServer : BUG detected : Can't store Upload!");
		c->closeConnection();
		delete c;
		delete u;
		return 0;
	}
	delete c;
	return 1;
}

int FServer::doScan()
{
	Path	*p;
	Path	*prv;
	DIR	*dp;	
	int	i;

	if (files)
	{
		files->clearDB();
		delete files;
		files	= 0;
	}

	if (cache)
		cache->clearDB();

	for (i = 0; i < (int)SEARCH_ARRAY_SIZE; ++i)
		if (searchArray[i])
		{
			free(searchArray[i]);
			searchArray[i]	= 0;
		}

	for (i = 0; i < (int)HASH_ARRAY_SIZE; ++i)
		if (hashArray[i])
		{
			free(hashArray[i]);
			hashArray[i]	= 0;
		}

	sizeShared	= 0;
	files		= 0;
	searchArraySize	= 0;
	hashArraySize	= 0;
	cacheHits	= 0;

	/*
	 * Remove all subdirs to speed things up. They'll be added
	 * in doScanRec anyway
	 */
	p	= prv	= paths;
	while (p)
	{
		if (p->isSubdir())
		{
			prv->setNext(p->getNext());
			if (p == prv)
			{
				p	= prv->getNext();
				delete prv;
				paths	= prv	= p;
			}
			else
			{
				delete p;
				p	= prv->getNext();			
			}
		}
		else
		{
			prv	= p;
			p	= p->getNext();
		}
	}
	for (p = paths; p; p = p->getNext())
	{
		char	tmp[100];

		if ((dp = opendir(p->getPathp())))
		{
			if (!files)
				files		= new DB(MCC->get_MAX_FILES_SHARED());
			if (!doScanRec(dp, p))
				break;
		}
		else
		{
			sprintf(tmp,
				"FServer : Opening directory FAILED (%s)!!!",
				strerror(errno));
			MCC->addError(tmp);
			return 0;
		}
	}

	/*
	 * Files have been inserted in a sorted order
	 * idx++ does that ....
	 * Call an expensive balance to optimize the DB.
	 */

	if (files)
	{
		int len;

		files->balance();
		len	= files->getElements();
		for(i = 0; i < len; ++i)
		{
			File	*f;

			if (!((f = (File *)files->findElement(i))))
				continue;
			addToHashArray(f->getPreparedFnp(), f->getEntry());
			addToSearchArray(f->getPreparedFnp(), f->getEntry());
		}
		return files->getElements();
	}

	return 0;
}

int FServer::doScanRec(DIR *dp, Path *p)
{
	struct dirent	*ep;
	char		pn[PATH_NAME_LENGTH];

	if (files->isFull())
	{
		closedir(dp);
		return 0;
	}

	p->getPath(pn);
	while (dp)
	{
		File 		*f;
		struct	stat	fst;
		DIR 		*newDp;
		char		fname[PATH_NAME_LENGTH];

		if ((ep = readdir(dp)))
		{
			if ( (!strcmp(ep->d_name, ".")) ||
					(!strcmp(ep->d_name, "..")) )
				continue;

			sprintf(fname, "%s/%s", pn, ep->d_name);
			if (!((newDp = opendir(fname))))
			{
				if (stat(fname, &fst) == -1)
				{
					char	damn[100];

					sprintf(damn,
						"FServer : stat failed (%s).",
						strerror(errno));
					MCC->addError(damn);
					continue;
				}

				f	= new File((int)(fst.st_size / 1024), ep->d_name, p);
				sizeShared	+= (int)(fst.st_size / 1024);
	
				if (files->storeElement(f) == ENTRY_INVALID)
					delete f;
				
				if (files->isFull())
				{
					closedir(dp);
					return 0;
				}
			}
			else
			{
				if (addPath(fname))
				{
					paths->setSubdir(1);
					if (!doScanRec(newDp, paths))
					{
						closedir(dp);
						return 0;
					}
				}
			}
		}
		else
			break;
	}
	closedir(dp);

	return 1;
}

int FServer::push(int fileID, unsigned int hostip,
					unsigned short hostport)
{
	Upload			*u;
	File			*f;
	Path			*p;
	struct  sockaddr_in     sa;
	int			pushSock;

	/*
	 * See if we can find that file....
	 */

	if (!files)
		return 0;
	if (!(f = (File *)files->findElement(fileID)))
		return 0;

	if (!(p = f->getPath()))
		return 0;

	sa.sin_family           = PF_INET;
        sa.sin_addr.s_addr      = hostip;
	if (!hostport)
	        sa.sin_port             = htons(6346);
	else        	
        	sa.sin_port             = htons(hostport);

	if ((pushSock = socket(PF_INET, SOCK_STREAM, 0)) < 0)
        {
                MCC->addError("FServer : ERROR OPENING SOCKET!!!");
		return 0;
	}

	u	= new Upload(pushSock, fileID, f->getFileNamep(), p->getPathp(), 0, sa,
					MCC->get_CLIENTIDp());
	u->setType(CON_TYPE_PUSH_OUT);
	u->setOutgoing();
	
	if ( (!u->Start(1)) || (uploads->storeElement(u) == ENTRY_INVALID))
	{
		delete u;
		return 0;
	}
	
	u->tryToConnect();
	return 1;
}

int FServer::pollFServer()
{
	return handleUploads();
}

int FServer::handleUploads()
{
	int	i;
	int	t;
	Upload	*u;

	if (!uploads->getElements())
		return 0;

	t	= 0;
	for (i = 0; i < uploads->getSize(); ++i)
	{
		if ( (u = (Upload *)uploads->findElement(i)) )
		{
			switch(u->getState())
			{
				case	CON_STATE_INIT		:
						u->tryToConnect();
						break;
				case	CON_STATE_CONNECTED	:
						u->loginToPeer();
						break;
				case	CON_STATE_LOGIN_SENT	:
				case	CON_STATE_ACTIVE 	:
						if (u->sendBlob() >= 0)
							t++;
						break;
				case	CON_STATE_DONE		:
						/*
						 * Stupid Workaround...
						 * Without sleep the last data
						 * is not received correctly
						 */
						sleep(1);
						uploads->deleteElement(i);
						break;
				default			:
						uploads->deleteElement(i);
					break;
			}
		}
	}

	return t;
}

int FServer::killUpload(int i)
{
	return uploads->deleteElement(i);
}

int FServer::addPath(char *newPath)
{
	Path	*np;
//	Path	*check;
	char	pn[PATH_NAME_LENGTH];

	if (newPath)
	{
		if (newPath[0] == '~')
		{
			sprintf(pn, "%s%s", getenv("HOME"), newPath + 1);
			newPath		= &pn[0];
		}
		/*
		 * Checking whether a path is already shared may seem
		 * desirable, but in increases the order of complexity
		 * from O(1) to O(n). For addind n dirs, that would
		 * mean from O(n) to O(n^2).
		 * That's not exeptable when sharing a lot of dirs.
		 */
/*
		for (check = paths; check; check = check->getNext())
		{
			char	checkName[PATH_NAME_LENGTH];
			
			check->getPath((char *)checkName);
			if (!strncmp(checkName, newPath, PATH_NAME_LENGTH))
				return 0;
		}
*/		
		np	= new Path(newPath);
		np->setNext(paths);
		paths	= np;
		return 1;
	}

	return 0;
}

int FServer::deletePath(char *oldPath)
{
	Path	*prv, *tmp;
	int	res;

	prv	= 0;
	res	= 0;
	for (tmp = paths; tmp; tmp = tmp->getNext())
	{
		if (strlen(tmp->getPathp()) < strlen(oldPath))
		{
			prv	= tmp;
			continue;
		}
		if (!strncmp(tmp->getPathp(), oldPath, strlen(oldPath)))
		{
			res	= 1;
			if (prv)
				prv->setNext(tmp->getNext());
			else
				paths	= tmp->getNext();
			delete tmp;
			if (prv)
				tmp	= prv;
			else
				tmp	= paths;

			if (files)
			{
				int	i;

				files->clearDB();
				delete(files);
				files	= 0;
				if (cache)
					cache->clearDB();

				for (i = 0; i < (int)SEARCH_ARRAY_SIZE; ++i)
					if (searchArray[i])
					{
						free(searchArray[i]);
						searchArray[i]	= 0;
					}
			}
		}		
		prv	= tmp;
	}
	return res;
}

int FServer::delAllPath()
{
	Path	*p, *prv;

	p	= paths;
	while (p)
	{
		prv	= p;
		p	= p->getNext();
		delete	prv;
	}
	paths	= 0;

	return 1;
}

TransferStatus *FServer::getStatusList()
{
	Upload		*u;
	TransferStatus	*prv, *current;
	int		i;

	current		= 0;
	for (i = 0; i < uploads->getSize(); i++)
		if ((u = (Upload *)uploads->findElement(i)))
		{
			prv	= current;
			current	= new TransferStatus(u);
			current->setNext(prv);
		}

	return current;
}

unsigned int FServer::getSizeShared()
{
	return sizeShared;
}

unsigned int FServer::getFilesCount()
{
	if (files)
		return (unsigned int)files->getElements();

	return 0;
}

unsigned int FServer::getLocalSearches()
{
	return localSearches;
}

unsigned int FServer::getCacheHits()
{
	return cacheHits;
}

unsigned int FServer::getCacheEntries()
{
	return cache->getElements();
}

unsigned int FServer::getCacheSize()
{
	return cache->getSize();
}

unsigned int FServer::getSearchArraySize()
{
	return searchArraySize;
}

unsigned int FServer::getHashArraySize()
{
	return hashArraySize;
}

int FServer::resize()
{
        DB      *newUploads;
        int     i;

        if (MCC->get_MAX_UPLOADS() < uploads->getElements())
	{
                MCC->addError("FServer : Too many uploads for "
                                "resize!");
                return 0;
        }

        newUploads    = new DB(MCC->get_MAX_UPLOADS());

        for (i = 0; (	(i < newUploads->getSize())	&&
			(i < uploads->getSize())	); i++)
                newUploads->storeElement(uploads->findElement(i));

        delete uploads;

        uploads       = newUploads;

        return 1;
}

int FServer::resizeCache()
{
	int	i;
	DB 	*newCache;

	if (MCC->get_CACHE_ENTRIES() <= 0)
	{
		cache->clearDB();
		delete cache;
		cache	= 0;
		return 0;
	}

	if (cache->getSize() == MCC->get_CACHE_ENTRIES())
		return 0;

	newCache	= new DB(MCC->get_CACHE_ENTRIES());

	for (i = 0; (i < cache->getSize()) && (i < newCache->getSize()); ++i )
	{
		CacheEntry	*ce;

		if ((ce	= (CacheEntry *)cache->findElement(i)))
		{
			cache->removeElement(i);
			if (newCache->storeElement(ce) == ENTRY_INVALID)
				delete ce;
		}
	}
	cache->clearDB();
	delete cache;
	cache	= newCache;
	return 1;
}

/*
 * This thing used practically all CPU power. Time for optimization!!!
 */

GnutellaPacket *FServer::mkSearchResp(char *findme)
{
	Packet		*p;
	LocalResult	*res, *wres;
	int		i;
	int		more;
	char		fn[FILE_NAME_LENGTH];
	char		host[256];
	CacheEntry	*ce;
	struct hostent	*h;
	unsigned int	*searchBits;

	if (!files)
		return 0;

	localSearches++;
	more	= preparePattern(findme);

	if (!more)
		return 0;

	// check cache
	if (cache)
	{
		ce	= new CacheEntry(findme);

		if (((i = cache->containsElement(ce))) != ENTRY_INVALID)
		{
			delete ce;
			cacheHits++;
			if ((ce = (CacheEntry *)cache->findElement(i)))
				return ce->getResponsep();
			else
				MCC->addError("BUG : FServer : cacheEntry expected but not found");

			return 0;
		}

		if (cache->storeElement(ce) == ENTRY_INVALID)
		{
			delete ce;
			ce	= 0;
			MCC->addError("BUG : Can't store CacheEntry");
			return 0;
		}
	}
	else
		ce	= 0;

	// start smart search
	searchBits	= 0;
	searchBits	= searchHashArray(findme, searchBits);
	if (!((searchBits = searchSearchArray(findme, searchBits))))
		return 0;

	p	= 0;
	/*
	 * Create LocalResult List
	 */
	res	= mkLocalResultList(searchBits, findme);
	free(searchBits);

	if (!res)
		return 0;

	// #7
	gethostname(host, 256);
	h	= gethostbyname(host);
	p	= new Packet(MCC);
	p->mkSearchRespPacket(MCC->get_CLIENTIDp(), MCC->get_PORT(),
				*(unsigned long int*) (h->h_addr_list[0]),
				MCC->get_SPEED());


	for (wres = res; wres; wres = wres->getNext())
	{
		(wres->getLRFile())->getFileName(fn);
		if (!p->addToSRPacket((wres->getLRFile())->getEntry(),
					(wres->getLRFile())->getSize(),
					fn))
 			break;
	}

	res->recDelLocalResult();

	if (ce)
		ce->setResponse(p);

	// #9
	return p;
}

int FServer::isUsingHost(int hi)
{
	int		i;
	Upload		*u;

	for (i = 0; i < uploads->getSize(); i++)
	{
		if ((u = (Upload *)uploads->findElement(i)))
			if (u->getHostIndex() == hi)
				return 1;
	}

	return 0;
}

unsigned int *FServer::ANDSearch(unsigned int *result, unsigned int *model,
					unsigned int range)
{
	unsigned int	i;
	int		res;

	if (!model)
	{
		if (result)
			free(result);
		return 0;
	}

	if (!result)
	{
		result	= (unsigned int *)malloc(range * sizeof(unsigned int));
		memcpy(result, model, range);
		return result;
	}

	res	= 0;
	for (i = 0; i < range; ++i)
		res = ( res	| 	((result[i]	= result[i] & model[i])) );

	if (!res)
	{
		free(result);
		return 0;
	}

	return result;
}

int FServer::addToSearchArray(char *fn, int idx)
{
	int	i;
	int	len;
	int 	sum;
	len	= strlen(fn);
	for (i = 0; i < len; ++i)
		if (strchr(VALIDCHARS, fn[i]))
		{
			sum	= VAL1(fn[i]);
			MKARRAY(searchArray, sum, searchArraySize);
   			SETBIT(searchArray[sum], idx);

 			if ( (fn[i + 1]) && (strchr(VALIDCHARS, fn[i + 1])) )
			{
				sum	= VAL2(fn[i], fn[i + 1]);
				MKARRAY(searchArray, sum, searchArraySize);
				SETBIT(searchArray[sum], idx);

 				if ( (fn[i + 2]) && (strchr(VALIDCHARS, fn[i + 2])) )
				{
					sum	= VAL3(fn[i], fn[i + 1], fn[i + 2]);
					MKARRAY(searchArray, sum, searchArraySize);
					SETBIT(searchArray[sum], idx);
				}
			}
		}

	return 1;
}

int FServer::preparePattern(char *buf)
{
	int	i;
	int	j;
	int	res;

	res	= 0;
	j	= 0;
	for (i = 0; buf[i] && (i < FILE_NAME_LENGTH); ++i)
	{
		buf[i + j]	= toupper(buf[i]);
		if (strchr(VALIDCHARS, buf[i + j]))
			res	= 1;
		else
		{
			buf[i + j]	= ' ';
			if (	( (i > 0) && (buf[i + j - 1] == ' ') ) 	||
				( (!i) 	&& (buf[i + j] == ' '))		)
				j--;
		}
	}
	if (buf[i + j - 1] == ' ')
		buf[i + j - 1]	= 0;
	else
		buf[i + j]	= 0;

	return res;
}

LocalResult *FServer::mkLocalResultList(unsigned int *searchBits, char *findme)
{
	LocalResult	*res;
	LocalResult	*wres;
	LocalResult	*prv;
	File		*f;
	char		*next;
	int		i;
	int		more;

	res	= 0;
	more	= 1;

	/*
	 * Isolate first pattern, next is beginning of the rest
	 */
	for (next = findme; 1; next++)
	{
		if (*next)
		{
			if (strchr(SEPARATORS, (int)*next))
			{
				*next	= 0;
				break;
			}
		}
		else
		{
			more	= 0;
			break;
		}
	}
	next++;

	for (i = 0; i < files->getElements(); i++)
	{
		// check to see if any bits in int are set
		if (!searchBits[ i / UIBITS ])
		{
			i	+= UIBITS - 1;
			continue;
		}

		// if so, find a bit
		if (!ISSET(searchBits, i))
			continue;

		// find the corresponding file
		if (!((f = (File *)files->findElement(i))))
		{
 			MCC->addError("BUG : Inconsistency in FileServer");
 			continue;
		}

		// And add it to our localresult list.
		if (strstr(f->getPreparedFnp(), findme))
			res	= new LocalResult(f, res);
	}

	if (!res)
		return 0;

	/*
	 * Scan list for the rest of the patterns
	 */
	prv	= 0;
	while(more)
	{
		findme	= next;

		/*
		 * Find end of pattern & next
		 */
		while(1)
		{
			if (*next)
			{
				if (strchr(SEPARATORS, (int)*next))
				{
					*next	= 0;
					break;
				}
				*next	= toupper(*next);
			}
			else
			{
				more	= 0;
				break;
			}
			next++;
		}
		next++;

		wres	= res;
		prv	= 0;
		while (wres)
		{
			if (strstr(wres->getFileNamep(), findme))
			{
				prv	= wres;
				wres	= wres->getNext();
			}
			else
			{
				/*
				 * Pattern not found, delete wres from list
				 */
				if (!prv)
				{
					prv	= wres;
					wres	= wres->getNext();
					res	= wres;
					delete	prv;
					prv	= 0;
				}
				else
				{
					wres	= wres->getNext();
					delete prv->getNext();
					prv->setNext(wres);
				}
			}
		}

		if (!res)
			more	= 0;
	}
	return res;
}

int FServer::addToHashArray(char *pattern, int idx)
{
	int		res;
	int		hv;
	unsigned char	pat[FILE_NAME_LENGTH];
	unsigned char	*c;
	unsigned char	*p;

	if (strlen(pattern) < 4)
		return 0;

	res	= 0;
	strcpy((char *)pat, pattern);
	p	= pat;

	while ((c = (unsigned char *)strchr((char *)p, ' ')))
	{
		int	l;

		*c	= 0;
		l	= strlen((char *)p);
		if (l > 6)
		{
			res	= 1;
			hv	= VAL3(p[0], p[3], p[6]);
			MKARRAY(hashArray, hv, hashArraySize);
			SETBIT(hashArray[hv], idx);
		}
		else if (l > 3)
		{
			res	= 1;
			hv	= VAL2(p[0], p[3]);
			MKARRAY(hashArray, hv, hashArraySize);
			SETBIT(hashArray[hv], idx);
		}
		p	+= l;
	}

	return res;
}

unsigned int *FServer::searchSearchArray(char *findme,
					unsigned int *searchBits)
{
	int		i;
	int		inc;
	unsigned int	beenThereDoneThat[SEARCH_ARRAY_SIZE / UIBITS];

 	for (i = 0; i < (int)(SEARCH_ARRAY_SIZE / UIBITS); ++i)
		beenThereDoneThat[i]	= 0;

	for (i = 0; i < (int)strlen(findme); i += inc)
	{
		int sum;

		inc	= 1;
		if  (strchr(VALIDCHARS, findme[i]))
		{
			if ( (findme[i + 1]) && (strchr(VALIDCHARS, findme[i + 1])) )
			{
				if ( (findme[i + 2]) && (strchr(VALIDCHARS, findme[i + 2])) )
				{
					sum	= VAL3(findme[i], findme[i + 1], findme[i + 2]);
					inc	= 3;
				}
				else
				{
					sum	= VAL2(findme[i], findme[i + 1]);
					inc	= 2;
				}
			}
			else
				sum	= VAL1(findme[i]);

			if (ISSET(beenThereDoneThat, sum))
				continue;

			if (! ((searchBits = ANDSearch(searchBits, searchArray[sum],
							(files->getElements() / UIBITS) + 1) )) )
				return 0;

			SETBIT(beenThereDoneThat, sum);
		}
	}
	return searchBits;
}

unsigned int *FServer::searchHashArray(char *findme,
					unsigned int *searchBits)
{
	unsigned char	pat[FILE_NAME_LENGTH];
	unsigned char	*c;
	unsigned char	*p;
	int		i;
	int		hv;
	unsigned int	beenThereDoneThat[HASH_ARRAY_SIZE / UIBITS];

	if (strlen(findme) < 4)
		return searchBits;

	strcpy((char *)pat, findme);
	p	= pat;

 	for (i = 0; i < (int)(HASH_ARRAY_SIZE / UIBITS); ++i)
		beenThereDoneThat[i]	= 0;

	while ((c = (unsigned char *)strchr((char *)p, ' ')))
	{
		int	l;

		*c	= 0;
		l	= strlen((char *)p);
		if (l > 6)
			hv	= VAL3(p[0], p[3], p[6]);
		else if (l > 3)
			hv	= VAL2(p[0], p[3]);
		else
		{
			p	+= l;
			continue;
		}

		if (hashArray[hv])
		{
			if (ISSET(beenThereDoneThat, hv))
			{
				p	+= l;
				continue;
			}

			if (! ((searchBits = ANDSearch(searchBits, hashArray[hv],
							(files->getElements() / UIBITS) + 1) )) )
				return 0;

			SETBIT(beenThereDoneThat, hv);
		}
		p	+= l;
	}

	return searchBits;
}
