/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include "Config.h"
#include "ControlCentre.h"
#include "DBElement.h"
#include "Buffer.h"
#include "GnutellaPacket.h"
#include "Connection.h"
#include "GnutellaConnection.h"

GnutellaConnection::GnutellaConnection()
{
	recvd		= 0;
	sent		= 0;
	dropped		= 0;
	errors		= 0;
}

GnutellaConnection::GnutellaConnection(Connection *c)
{
	if (inBuffer)
		delete inBuffer;
	if (outBuffer)
		delete outBuffer;
	connection	= c->getConnection();
	hostIndex	= c->getHostIndex();
	outgoing	= c->isOutgoing();
	timeStamp	= c->getTimeStamp();
	type		= c->getType();
	inBuffer	= c->copyInBuffer();
	outBuffer	= c->copyOutBuffer();
	recvd		= 0;
	sent		= 0;
	dropped		= 0;
	errors		= 0;
}

int GnutellaConnection::sendPacket(GnutellaPacket *p)
{
	if (outBuffer->bytesEmpty() <
			(GNUTELLA_HEADER_LENGTH + p->getDataLength()))
		return 0;

	outBuffer->add(p->getHeaderp(), GNUTELLA_HEADER_LENGTH);
	if (p->getDataLength() > 0)
		outBuffer->add(p->getBodyp(), p->getDataLength());

	return 1;
}

int GnutellaConnection::getSent()
{
        return sent;
}

int GnutellaConnection::getRecvd()
{
        return recvd;
}

int GnutellaConnection::getDropped()
{
        return dropped;
}

int GnutellaConnection::getErrors()
{
        return errors;
}

void GnutellaConnection::punchSent()
{
	sent++;
}

void GnutellaConnection::punchRecvd()
{
	recvd++;
}

void GnutellaConnection::punchDropped()
{
	dropped++;
}

void GnutellaConnection::punchErrors()
{
	errors++;
}
