#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "Config.h"
#include "ControlCentre.h"
#include "DBElement.h"
#include "DB.h"
#include "Buffer.h"
#include "GnutellaPacket.h"
#include "Connection.h"
#include "Host.h"
#include "Mesg.h"
#include "SearchResult.h"
#include "Packet.h"
#include "SearchManager.h"
#include "GnutellaRouter.h"

SearchManager::SearchManager(ControlCentre *newMCC)
{
	MCC		= newMCC;
	signature	= 0;
	results		= 0;
}

SearchResult *SearchManager::getSearchResult(int i)
{
	if (!results)
		return 0;
	else
		return (SearchResult *)results->findElement(i);
}

void SearchManager::clearResults()
{
	if (results)
	{
		results->clearDB();
		delete(results);
		results	= 0;
	}
}

int SearchManager::newQuery(int spd, Mesg *sig)
{
	if (MCC->get_AUTO_CLEAR_RESULTS())
		clearResults();

	results		= new DB(MCC->get_MAX_RESULTS());
	signature	= sig;
	speed		= spd;

	return 1;
}

int SearchManager::addResults(GnutellaPacket *p)
{
	int		res;
	if (!signature)
		return 0;

	if (!results)
	{
		MCC->addError("BUG : SearchManager : No results DB!!!");
		return 0;
	}


	if (results->isFull())
		return 0;

	if (p->getDataLength() > MAX_SEARCH_RESP_BODY_SIZE)
		return 0;

	res	= bodyToResults(p->getBodyp(), p->getDataLength(), p->getConnection());

	return res;
}

int SearchManager::bodyToResults(char *body, unsigned int len, int ci)
{
 	unsigned char	numRecs;
	short		hostPort;
	unsigned int	hostIP;
	int		hostSpeed;
	int		index;
	int		size;
	char		name[FILE_NAME_LENGTH];
	char		GClientID[CLIENTID_LENGTH];
        int     	i;

        /*
         * I think this is the fastest way of copying all members
         * from an array of bytes. Feel free to disagree.
         */

//		PARAMETER	BODY + OFFSET	RANGE
//	      |---------------|--------------|-------|
        memcpy(&numRecs,	body, 		1);
	memcpy(&hostPort,	body + 1, 	2);
	memcpy(&hostIP, 	body + 3, 	4);
	memcpy(&hostSpeed, 	body + 7, 	4);
/*
fprintf(stderr, "Results : numrecs %d, hostport %d hostip %d, "
		"hostspeed %d.\n", numRecs, hostPort, hostIP, hostSpeed);
*/
        memcpy(GClientID, (body + len - CLIENTID_LENGTH), CLIENTID_LENGTH);

	if (hostSpeed < speed)
		return 0;

	// skip to Resultrecords
	body	+= 11;
        for (i = 0; i < numRecs; i++)
	{
		SearchResult	*s;

	        memcpy(&index, body, 4);
        	memcpy(&size, body + 4, 4);
		body	+= 8;

                body  	+= readResultNameFromString(body, (char *)name);

		s	= new SearchResult(hostPort, hostIP, hostSpeed,
				index, size, ci, name, GClientID);

		if (results->storeElement(s) == ENTRY_INVALID)
			delete s;
		else
			MCC->setNotifyResults();
	}

	return numRecs;
}

int SearchManager::readResultNameFromString(char *buf, char *name)
{
        int     i;

        for (i = 0; (buf[i] && ( i < FILE_NAME_LENGTH - 1) ); i++)
                if (buf[i] || buf[i + 1])
                        name[i] = buf[i];

        name[i++]       = 0;
        name[i]         = 0;

        return i + 1;
}

/*
 * FAME
 */

Mesg *SearchManager::getSignature()
{
	return signature;
}

